@@ -1,12 +1,20 @@
 """pydantic models for GeoJSON Geometry objects."""
 
 import abc
-from typing import Any, List, Tuple, Union
+from typing import Any, List, Union
 
 from pydantic import BaseModel, Field, ValidationError, validator
 from pydantic.error_wrappers import ErrorWrapper
 
-from .utils import NumType
+from geojson_pydantic.types import (
+    LineStringCoords,
+    MultiLineStringCoords,
+    MultiPointCoords,
+    MultiPolygonCoords,
+    NumType,
+    PolygonCoords,
+    Position,
+)
 
 
 class _GeometryBase(BaseModel, abc.ABC):
@@ -19,60 +27,65 @@ class _GeometryBase(BaseModel, abc.ABC):
         return self.dict()
 
 
-Coordinate = Union[Tuple[NumType, NumType], Tuple[NumType, NumType, NumType]]
-
-Position = Coordinate
-
-
 class Point(_GeometryBase):
     """Point Model"""
 
     type: str = Field("Point", const=True)
-    coordinates: Coordinate
+    coordinates: Position
 
 
 class MultiPoint(_GeometryBase):
     """MultiPoint Model"""
 
     type: str = Field("MultiPoint", const=True)
-    coordinates: List[Coordinate]
+    coordinates: MultiPointCoords
 
 
 class LineString(_GeometryBase):
     """LineString Model"""
 
     type: str = Field("LineString", const=True)
-    coordinates: List[Coordinate] = Field(..., min_items=2)
+    coordinates: LineStringCoords
 
 
 class MultiLineString(_GeometryBase):
     """MultiLineString Model"""
 
     type: str = Field("MultiLineString", const=True)
-    coordinates: List[List[Coordinate]]
+    coordinates: MultiLineStringCoords
 
 
 class Polygon(_GeometryBase):
     """Polygon Model"""
 
     type: str = Field("Polygon", const=True)
-    coordinates: List[List[Coordinate]]
+    coordinates: PolygonCoords
 
     @validator("coordinates")
-    def check_coordinates(cls, coords):
-        """Validate that Polygon coordinates pass the GeoJSON spec"""
-        if any([len(c) < 4 for c in coords]):
-            raise ValueError("All linear rings must have four or more coordinates")
-        if any([c[-1] != c[0] for c in coords]):
+    def check_closure(cls, polygon):
+        """Validate that Polygon is closed (first and last coordinate are the same)."""
+        if any([ring[-1] != ring[0] for ring in polygon]):
             raise ValueError("All linear rings have the same start and end coordinates")
-        return coords
+
+        return polygon
+
+    @classmethod
+    def from_bounds(
+        cls, xmin: NumType, ymin: NumType, xmax: NumType, ymax: NumType
+    ) -> "Polygon":
+        """Create a Polygon geometry from a boundingbox."""
+        return cls(
+            coordinates=[
+                [[xmin, ymin], [xmin, ymax], [xmax, ymax], [xmax, ymin], [xmin, ymin]]
+            ]
+        )
 
 
 class MultiPolygon(_GeometryBase):
     """MultiPolygon Model"""
 
     type: str = Field("MultiPolygon", const=True)
-    coordinates: List[List[List[Coordinate]]]
+    coordinates: MultiPolygonCoords
 
 
 Geometry = Union[Point, MultiPoint, LineString, MultiLineString, Polygon, MultiPolygon]
@@ -104,10 +117,10 @@ def parse_geometry_obj(obj) -> Geometry:
     """
     if "type" not in obj:
         raise ValidationError(
-            [
-                ErrorWrapper(ValueError("Missing 'type' field in geometry"), "type"),
-                "Geometry",
-            ]
+            errors=[
+                ErrorWrapper(ValueError("Missing 'type' field in geometry"), loc="type")
+            ],
+            model=_GeometryBase,
         )
     if obj["type"] == "Point":
         return Point.parse_obj(obj)
